﻿'use strict';
const Gulp = require('gulp');
const Del = require('del');
const Path = require('path');
const RunSequence = require('run-sequence');
const _ = require('lodash');
const Merge2 = require('merge2');
const ReworkPluginUrl = require('rework-plugin-url')

const Plugins = require('../util/plugins');
const TasksUtils = require('../util/utils');
const Config = TasksUtils.gulpConfig;
const Metalsmith = require('../util/metalsmith');
const StaticAssets = require('../util/staticAssets');


// Compress the files
Gulp.task('compile', function(callback) {
  RunSequence('compile-clean',
    'compile-images', // needs to be before css, since we add cache busters to image urls
    'compile-css', // sometimes hits a timeout when run in parallel with other tasks
    ['compile-js', 'compile-copy'],
    'compile-static-site',
    callback);
});

Gulp.task('compile-clean', function() {
  return Del([
    Config.compile.dir
  ]);
});

Gulp.task('compile-css', function() {
  if (!Config.compile.css) return;

  return Gulp.src(Config.compile.css.src, { cwd: Config.build.dir })
    .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))
    .pipe(Plugins.rework(ReworkPluginUrl(function (url) {
      return StaticAssets.cssUrlWithRev(url);
    })))
    .pipe(Plugins.cleanCss(Config.compile.css.settings))
    .pipe(Plugins.rev())
    .pipe(Gulp.dest(Config.compile.css.dest))
    .pipe(Plugins.filesize())
    .pipe(Plugins.rev.manifest('css-manifest.json'))
    .pipe(Gulp.dest('compile/rev'));
});

Gulp.task('compile-js', function() {
  if (!Config.compile.js) return;

  return Gulp.src(Config.compile.js.src, { cwd: Config.build.dir })
    .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))
    .pipe(Plugins.uglify(Config.compile.js.settings))
    .pipe(Plugins.rev())
    .pipe(Gulp.dest(Config.compile.js.dest))
    .pipe(Plugins.filesize())
    .pipe(Plugins.rev.manifest('js-manifest.json'))
    .pipe(Gulp.dest('compile/rev'));
});

Gulp.task('compile-images', function() {
  const bundles = TasksUtils.arrayOrEmpty(Config.compile.images);
  if (!bundles) return;

  const streams = bundles.map((bundle) => {
    let stream = Gulp.src(bundle.src, bundle.srcOpts)
        .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))
        .pipe(Plugins.imagemin(bundle.settings));

    if (bundle.addRev) {
      stream = stream
          .pipe(Plugins.rev());
    }

    stream = stream
        .pipe(Gulp.dest(bundle.dest))
        .pipe(Plugins.filesize());

    return stream;
  });

  return Merge2.apply(null, streams)
      .pipe(Plugins.rev.manifest('images-manifest.json'))
      .pipe(Gulp.dest('compile/rev'));
});

Gulp.task('compile-copy', function() {
  const bundles = TasksUtils.arrayOrEmpty(Config.compile.copy);
  if (!bundles) return;

  const streams = bundles.map((bundle) => {
    return Gulp.src(bundle.src, bundle.srcOpts)
        .pipe(Plugins.plumber(TasksUtils.streamErrorHandler))
        .pipe(Gulp.dest(bundle.dest));
  });
  return Merge2.apply(null, streams);
});


/**
 * TODO add html minification here and in production
 .pipe(plugins.minifyHtml(config.compile.markup.minifyHtmlOptions))
      minifyHtmlOptions: {
        empty: true,
        spare: true,
        quotes: true,
        conditionals: true  // KEEP conditional internet explorer comments
      }
 */
Gulp.task('compile-static-site', function(done) {
  Config.compile.getMarkupData().then(function(markupData) {
    markupData = _.assign(markupData, {
      baseUrl: '/compile',
      staticUrl: (url) => { return `/compile/${StaticAssets.filenameWithRev(url)}`; }
    });

    Metalsmith.build(markupData)
      .destination(Config.compile.dir)
      .build(function(err) {
        if (err) {
          TasksUtils.errorHandler(err);
        }
        done(err);
      });
  }).catch(done);
});
